@@ -19,15 +19,15 @@ namespace caching {
 	class metadata {
 	public:
 		enum open_type {
-			CREATE,
-			OPEN
+			CREATE
 		};
 
 		typedef block_manager<>::read_ref read_ref;
 		typedef block_manager<>::write_ref write_ref;
 		typedef boost::shared_ptr<metadata> ptr;
 
-		metadata(block_manager<>::ptr bm, open_type ot);
+		metadata(block_manager<>::ptr bm, open_type ot, unsigned metadata_version = 2); // Create only
+		metadata(block_manager<>::ptr bm);
 
 		void commit(bool clean_shutdown = true);
 		void setup_hint_array(size_t width);
@@ -40,11 +40,12 @@ namespace caching {
 		mapping_array::ptr mappings_;
 		hint_array::ptr hints_;
 		persistent_data::bitset::ptr discard_bits_;
+		persistent_data::bitset::ptr dirty_bits_;
 
 	private:
 		void init_superblock();
 
-		void create_metadata(block_manager<>::ptr bm);
+		void create_metadata(block_manager<>::ptr bm, unsigned metadata_version);
 		void open_metadata(block_manager<>::ptr bm);
 
 		void commit_space_map();
