@@ -2,6 +2,7 @@
 #include <getopt.h>
 #include <libgen.h>
 
+#include "base/output_file_requirements.h"
 #include "persistent-data/file_utils.h"
 #include "thin-provisioning/commands.h"
 #include "human_readable_format.h"
@@ -23,8 +24,12 @@ namespace {
 			emitter::ptr e = create_restore_emitter(new_md);
 
 			block_manager<>::ptr old_bm = open_bm(old_path, block_manager<>::READ_ONLY);
-			metadata::ptr old_md(new metadata(old_bm));
-			metadata_dump(old_md, e, true);
+
+			metadata::ptr old_md(new metadata(old_bm, false));
+
+			dump_options opts;
+			opts.repair_ = true;
+			metadata_dump(old_md, e, opts);
 
 		} catch (std::exception &e) {
 			cerr << e.what() << endl;
@@ -98,7 +103,10 @@ thin_repair_cmd::run(int argc, char **argv)
 		return 1;
 	}
 
-	if (!output_path) {
+	if (output_path)
+		check_output_file_requirements(*output_path);
+
+	else {
 		cerr << "no output file provided" << endl;
 		usage(cerr);
 		return 1;
