@@ -16,6 +16,8 @@
 // with thin-provisioning-tools.  If not, see
 // <http://www.gnu.org/licenses/>.
 
+#include "base/file_utils.h"
+#include "base/output_file_requirements.h"
 #include "persistent-data/file_utils.h"
 #include "thin-provisioning/commands.h"
 #include "thin-provisioning/emitter.h"
@@ -43,15 +45,20 @@ using namespace thin_provisioning;
 
 namespace {
 	int restore(string const &backup_file, string const &dev, bool quiet) {
+		bool metadata_touched = false;
 		try {
 			// The block size gets updated by the restorer.
 			block_manager<>::ptr bm(open_bm(dev, block_manager<>::READ_WRITE));
+			file_utils::check_file_exists(backup_file);
+			metadata_touched = true;
 			metadata::ptr md(new metadata(bm, metadata::CREATE, 128, 0));
 			emitter::ptr restorer = create_restore_emitter(md);
 
 			parse_xml(backup_file, restorer, quiet);
 
 		} catch (std::exception &e) {
+			if (metadata_touched)
+				file_utils::zero_superblock(dev);
 			cerr << e.what() << endl;
 			return 1;
 		}
@@ -138,7 +145,8 @@ thin_restore_cmd::run(int argc, char **argv)
 		cerr << "No output file provided." << endl << endl;
 		usage(cerr);
 		return 1;
-	}
+	} else
+		check_output_file_requirements(output);
 
 	return restore(input, output, quiet);
 }
